% This example computes both strain and polarization for an actuator using
% the 90 degree switching model, with a different quadrature rule, a
% different time step, and a smaller grid for evaluating the integration.
clear all;
addpath('../src/');

% Input field data loaded from an external file
filedata = load('example_02_input_data.mat');
data = getfield(filedata, 'York_PZT_10p6Mpa_Sampled');
nstep = 10;
E = data.E(1:nstep:end) * 1e6;
% This is the time vector
t = data.T(1:nstep:end);

% This says to compute the look-up tables used in updating the using this 
% time-step.
param.delta_t = t(2) - t(1);

% This says we are using the quadrature rule defined in 'quad_loglinear.m'
param.quadrature = 'quad_loglinear';

% This tells the initialization routine to compute both strain and 
% polarization in the 'forward' function using the routine in 
% 'eval_90_strain_polar.m'.
param.evaluation = 'eval_90_strain_polar';

% This says we will use 20 points for the coercive and interaction field
% grids.
param.Nc = 20;
param.Ni = 20;

hem = hemtool(param);

% Get both the strain and polarization
[eps, P, hem] = hem.forward(hem, E);


% Plot strain versus input electric field
figure(1);
plot(E/1e6, 100*eps);
legend('strain');
title('Strain versus Electric Field');
xlabel('Electric Field (MV/m)');
ylabel('Strain (%)');

% Plot polarization versus input electric field
figure(2);
plot(E/1e6, P);
legend('polarization');
title('Polarization versus Electric Field');
xlabel('Electric Field (MV/m)');
ylabel('Polarization (C/m^2)');

% Plot strain versus polarization
figure(3);
plot(P, 100*eps);
legend('strain');
title('Strain versus Polarization');
xlabel('Polarization (C/m^2)');
ylabel('Strain (%)');

